import json
import requests

from django.conf import settings


class PodleHttpClient:
    @staticmethod
    def make_request(url, method, data=None):
        # Build header
        access_token = settings.PODLE_AUTH_TOKEN
        headers = {
            "content-type": "application/json",
            "Authorization": "Bearer {}".format(access_token),
        }

        # Make HTTP request
        try:
            response = requests.request(method, url, json=data, headers=headers)
            result = response.text
        except Exception as e:
            raise e

        # Parse and return JSON Result
        try:
            return json.loads(result)
        except json.JSONDecodeError:
            return None


class PodleHelper:
    """
    Podle helper to interact the API.
    https://docs.podle.io/
    """

    client = PodleHttpClient()
    base_url = "https://api.podle.io/v1"
    endpoints = {
        "newsletters": f"{base_url}/newsletters",
        "dictionaries": f"{base_url}/dictionaries/default/words",
        "rss": f"{base_url}/rss",
    }

    def create_newsletter(self, data):
        url = self.endpoints["newsletters"]
        return self.client.make_request(url, "POST", data)

    def retrieve_newsletter(self, newsletter_id):
        url = f"{self.endpoints['newsletters']}/{newsletter_id}"
        return self.client.make_request(url, "GET")

    def create_or_update_word(self, data):
        url = self.endpoints["dictionaries"]
        return self.client.make_request(url, "POST", data)

    def delete_word(self, word):
        url = f"{self.endpoints['dictionaries']}/{word}"
        return self.client.make_request(url, "DELETE")

    def lookup_word(self, word):
        url = f"{self.endpoints['dictionaries']}/{word}"
        return self.client.make_request(url, "GET")

    def lookup_all_words(self):
        url = self.endpoints["dictionaries"]
        return self.client.make_request(url, "GET")

    def get_private_rss(self, subscriber_id, newsletter_name):
        url = f"{self.endpoints['rss']}?subscriberId={subscriber_id}&newsletterName={newsletter_name}"
        return self.client.make_request(url, "GET")

    def create_private_rss(self, data):
        url = self.endpoints["rss"]
        return self.client.make_request(url, "POST", data)

    def delete_private_rss(self, subscriber_id, newsletter_name):
        url = f"{self.endpoints['rss']}?subscriberId={subscriber_id}&newsletterName={newsletter_name}"
        return self.client.make_request(url, "DELETE")



from django.contrib import admin
from django.contrib.auth import get_user_model

from .models import Dictionary, Newsletter, RssFeed

User = get_user_model()


class NewsletterAdmin(admin.ModelAdmin):
    pass


class DictionaryAdmin(admin.ModelAdmin):
    search_fields = ("word", "pronunciation")
    list_display = ("word", "pronunciation")


class RssFeedAdmin(admin.ModelAdmin):
    list_display = ("user", "feed")
    readonly_fields = ("feed",)
    raw_id_fields = ("user",)

    def get_search_fields(self, request):
        return {"feed", f"user__{User.USERNAME_FIELD}"}


admin.site.register(Newsletter, NewsletterAdmin)
admin.site.register(Dictionary, DictionaryAdmin)
admin.site.register(RssFeed, RssFeedAdmin)



import logging

from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group
from django.db.models.signals import m2m_changed, pre_save, pre_delete, post_save
from django.dispatch import receiver

from ..models import Dictionary, RssFeed
from ..podle import PodleHelper

logger = logging.getLogger(__name__)
User = get_user_model()


@receiver(pre_save, sender=Dictionary)
def create_or_update_dictionary_word(sender, instance, *args, **kwargs):
    response = PodleHelper().create_or_update_word(
        {"value": instance.pronunciation, "raw": instance.word}
    )
    if "added" not in response:
        logger.error(response)
        raise Exception(response)

    logger.info(response)


@receiver(pre_delete, sender=Dictionary)
def delete_dictionary_word(sender, instance, *args, **kwargs):
    response = PodleHelper().delete_word(instance.word)
    if "deleted" not in response:
        logger.error(response)
        raise Exception(response)

    logger.info(response)


@receiver(post_save, sender=RssFeed)
def create_rss_feed(sender, instance, created, *args, **kwargs):
    if not instance.feed:
        response = PodleHelper().create_private_rss(
            {
                "subscriberId": instance.user.pk,
                "newsletterName": settings.PODLE_NEWSLETTER_NAME,
            }
        )

        feed = response.get(str(instance.user.pk), None)

        if not feed:
            logger.error(response)
            raise Exception(response)

        instance.feed = feed
        instance.save()

        logger.info(response)


@receiver(signal=m2m_changed, sender=User.groups.through)
def handle_rss_feeds(instance, action, reverse, model, pk_set, using, *args, **kwargs):
    """Create or delete private rss feeds when adding users to the group defined in PODLE_RSS_FEED_GROUP_NAME"""

    # signal can have User (reverse=False) or Group (reverse=True) as sender
    group_pks = [instance.pk] if reverse else pk_set

    if action == "post_add" or action == "post_remove":
        if not Group.objects.filter(
            pk__in=group_pks, name=settings.PODLE_RSS_FEED_GROUP_NAME
        ).exists():
            return False

        users_pks = pk_set if reverse else [instance.pk]

        if action == "post_add":
            for user_pk in users_pks:
                RssFeed.objects.get_or_create(user_id=user_pk)
        else:
            rss_feeds = RssFeed.objects.filter(user_id__in=users_pks)
            rss_feeds.delete()


@receiver(pre_delete, sender=RssFeed)
def delete_rss_feed(sender, instance, *args, **kwargs):
    response = PodleHelper().delete_private_rss(
        instance.user.pk, settings.PODLE_NEWSLETTER_NAME
    )
    if "deleted" not in response.get(str(instance.user.pk)):
        logger.error(response)
        raise Exception(response)

    logger.info(response)



import logging

from django.contrib.contenttypes.models import ContentType
from django.db import models

from .podle import PodleHelper

logger = logging.getLogger(__name__)


class NewsletterManager(models.Manager):
    def create_or_update_newsletter(self, instance, json_content):
        content_type = ContentType.objects.get_for_model(instance._meta.model)
        object_id = instance.id

        try:
            newsletter = self.get(content_type=content_type, object_id=object_id)
        except self.model.DoesNotExist:
            response = PodleHelper().create_newsletter(json_content)
            logger.debug(response)
            newsletter_id = response["id"]
            return self.create(
                content_type=content_type, object_id=object_id, uuid=newsletter_id
            )

        # If we update the newsletter, it is best to reuse the newsletter uuid
        json_content["newsletterId"] = str(newsletter.uuid)
        PodleHelper().create_newsletter(json_content)
        return newsletter


class RssFeedManager(models.Manager):
    def get_rss_feed(self, user):
        try:
            return self.get(user=user).feed
        except self.model.DoesNotExist:
            return None



from rest_framework import serializers


class NewsletterSerializer(serializers.Serializer):
    newsletter_url = serializers.CharField()

    def update(self, instance, validated_data):
        instance.audio_url = validated_data.get("newsletter_url", instance.audio_url)
        instance.save()
        return instance



from django.db import models

from django.contrib.auth import get_user_model
from django.contrib.contenttypes.models import ContentType

from .managers import NewsletterManager, RssFeedManager

User = get_user_model()


class Newsletter(models.Model):
    uuid = models.UUIDField()
    audio_url = models.URLField(blank=True, null=True, max_length=2000)
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    object_id = models.PositiveIntegerField()

    objects = NewsletterManager()

    class Meta:
        unique_together = ["content_type", "object_id"]

    def __str__(self):
        return f"{self.content_type} - {self.object_id}"


class Dictionary(models.Model):
    word = models.CharField(max_length=254, unique=True)
    pronunciation = models.CharField(max_length=254)

    class Meta:
        ordering = ["word"]
        verbose_name_plural = "Dictionaries"

    def __str__(self):
        return f"{self.word} - [{self.pronunciation}]"


class RssFeed(models.Model):
    user = models.OneToOneField(User, on_delete=models.CASCADE)
    feed = models.URLField(blank=True, null=True)

    objects = RssFeedManager()

    def __str__(self):
        return f"{self.user} - {self.feed}"



# Generated by Django 2.2.24 on 2021-06-11 14:28

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
    ]

    operations = [
        migrations.CreateModel(
            name="Dictionary",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("word", models.CharField(max_length=254, unique=True)),
                ("pronunciation", models.CharField(max_length=254)),
            ],
            options={
                "verbose_name_plural": "Dictionaries",
                "ordering": ["word"],
            },
        ),
        migrations.CreateModel(
            name="Newsletter",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("uuid", models.UUIDField()),
                ("audio_url", models.URLField(blank=True, max_length=2000, null=True)),
                ("object_id", models.PositiveIntegerField()),
                (
                    "content_type",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="contenttypes.ContentType",
                    ),
                ),
            ],
            options={
                "unique_together": {("content_type", "object_id")},
            },
        ),
    ]



from setuptools import setup, find_packages

from os import path

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="django-podle",
    version="0.4.0",
    description="Integration with Podle.io podcast app",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/briefmnews/django-podle",
    author="Brief.me",
    author_email="tech@brief.me",
    license="MIT",
    packages=find_packages(exclude=["tests"]),
    python_requires=">=3.7",
    install_requires=[
        "Django>=2.2",
        "djangorestframework>=3",
        "requests>=2",
    ],
    classifiers=[
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 2.2",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    include_package_data=True,
    zip_safe=False,
)



import logging

from django.shortcuts import get_object_or_404
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt

from rest_framework import status
from rest_framework.response import Response
from rest_framework.views import APIView

from .signals import audio_file
from .models import Newsletter
from .serializers import NewsletterSerializer

logger = logging.getLogger(__name__)


@method_decorator(csrf_exempt, name="dispatch")
class NewsletterWebhook(APIView):
    serializer_class = NewsletterSerializer

    def post(self, request, *args, **kwargs):
        newsletter_id = request.data.get("id", None)
        newsletter = get_object_or_404(Newsletter, uuid=newsletter_id)
        serializer = self.serializer_class(newsletter, data=request.data)
        if serializer.is_valid():
            serializer.save()
            logger.info(
                f"Audio file url saved successfully for newsletter {newsletter.uuid}"
            )
            audio_file.send(
                sender=Newsletter,
                success=True,
                audio_url=serializer.validated_data["newsletter_url"],
                newsletter=newsletter,
            )
            return Response(data={"success": True}, status=status.HTTP_200_OK)

        audio_file.send(sender=Newsletter, success=False, errors=serializer.errors)

        return Response(data=serializer.errors, status=status.HTTP_400_BAD_REQUEST)
